// ----------------------------------------------------------------------------
// Projekt       : Analytica AnaGate API
// File          : AnaGateClient.hpp
// Author        : Axel Schmidt
// Copyright     : (C) 2004 by Analytica GmbH
// ----------------------------------------------------------------------------
// $Id: AnaGateClient.hpp,v 1.16 2008/06/26 07:48:29 tobiashanss Exp $
//
// $Log: AnaGateClient.hpp,v $
// Revision 1.16  2008/06/26 07:48:29  tobiashanss
// Separaten Schalter fr das Packing eingefgt (USE_DEFAULT_PACKING).
//
// Revision 1.15  2007/11/12 10:50:21  axelschmidt
// Byte-Alignment
//
// Revision 1.14  2007/05/31 10:59:05  axelschmidt
// Empfangene Responses werden aus der Map nach 10*timeout gelscht, wenn sie nicht mehr verwendet werden knnen.
//
// Revision 1.13  2005/11/18 16:10:52  axelschmidt
// default parameter for connect function
//
// Revision 1.12  2005/09/16 09:18:49  axelschmidt
// Digital IO support for I2C and CAN
//
// Revision 1.11  2005/08/02 13:46:33  axelschmidt
// current MsgID not static data
//
// Revision 1.10  2005/08/01 16:52:32  axelschmidt
// virtual function for ConInd
//
// Revision 1.9  2005/07/22 16:19:23  AxelSchmidt
// open/close are global functions
//
// Revision 1.8  2005/07/22 07:13:50  AxelSchmidt
// new Info-function for all clients
//
// Revision 1.7  2005/07/11 10:54:30  axelschmidt
// gets TCPIP-State
//
// Revision 1.6  2005/06/07 16:36:58  axelschmidt
// new DataIndRespond-function
//
// Revision 1.5  2004/08/13 12:55:05  AxelSchmidt
// method Info implemented
//
// Revision 1.4  2004/08/06 07:00:15  AxelSchmidt
// Methode Info fr alle Devices
//
// Revision 1.3  2004/07/21 14:13:56  AxelSchmidt
// CAnaTCPIPBase not longer base class of CAnaGateClient
//
// Revision 1.2  2004/07/14 14:58:29  AxelSchmidt
// Fulltrace durch Debug Level 2 ersetzt
//
// Revision 1.1  2004/07/01 16:28:58  AxelSchmidt
// initial version
//
// ----------------------------------------------------------------------------

#ifndef _ANAGATE_CLIENT_HPP
#define _ANAGATE_CLIENT_HPP

// Includes -------------------------------------------------------------------
#include <AnaCommon.h>
#include <AnaTCPIPBase.h>
#include <map>

#include "AnaGate.h"
#include "AnaGateMsg.hpp"

#ifndef USE_DEFAULT_PACKING
// packing mode 1 Byte
#pragma pack(push,1)
#endif

/** Standard communication to an device of the AnaGate series via TCP/IP.
    @author  Axel Schmidt
    @version 1.0
*/
class CAnaGateClient 
{
   DECLARE_ANATRACE( CAnaGateClient );
   
public:
      typedef struct SAnaGateDeviceInfo SDeviceInfo;
      
      /** Standard construktor. 
         @param nDevice  Device type flag.
         @param nTimeout Standard timeout for send/receive requests (default = infinite ).
      */
      CAnaGateClient( EAnaGateDevice nDevice, int nTimeout = CAnaTCPIPBase::WAIT_INFINITE );

      /** Destructor */
      virtual ~CAnaGateClient( void );

      /** Establishes a TCP/IP connection to an AnaGate device. 
          @param psIPAddress TCP/IP address of AnaGate device (format: "a.b.c.d" or "host.domain.com").
          @param nPort  Defines the TCP/IP port on which the AnaGate device is listening for connection requests.
                        Uses 0 as default, which means that the standard port is to be used.
          @remarks Is the connection to the AnaGate device not established within the class timeout, the
                     the method fails.
          @return  Returns 0, if method succeeds. For valid return codes see #CAnaTCPIPBase::ERetCode.
      */
      int  Connect( const char *psIPAddress, const AnaUInt16 nPort = 0 );

      /** Disconnect the active connetion to an AnaGate device.
          @return  Returns 0, if method succeeds. For valid return codes see #CAnaTCPIPBase::ERetCode.
       */
      int Disconnect( void );

      /** Sends a standard AnaGate e telegramm and waits for response (confirmation). 
          @remarks Make sure that the returned memory is freed via delete by caller.
          @param  oMessage AnaGate telegram.
          @param  pResponse Incoming AnaGate telegram (NULL if no response, together with a timeout return code).
          @return return code (0, if successful).
      */
      int SendMsg( const CAnaGateMsg  & oMessage, CAnaGateMsg * & pResponse );

      /** Sends a standard AnaGate e telegramm and waits for response (confirmation). 
          @remarks The implementation of the method is a bit slower as the #SendMsg method with the reference pointer, 
                   because the #CAnaGateMsg structure has be copied from the intern representation
                   to the response parameter.
          @param  oMessage AnaGate telegram.
          @param  oResponse Incoming AnaGate telegram (is not valid, if timeout occured).
          @return return code (0, if successful).
      */
      int SendMsg( const CAnaGateMsg  & oMessage, CAnaGateMsg & oResponse );

      /** Search for a message with given id. If found a pointer to the message is returned, and the
          message is removed from the map. If not found NULL is returned.
          @remarks Make sure that the returned memory is freed via delete by caller.
          @param nMsgID message id
          @return pointer to found message, NULL otherwise.
       */
      CAnaGateMsg * GetMsg( short nMsgID );

      /** Waits for incoming telegramm with the given message id.
          @remarks Make sure that the returned memory is freed via delete by caller.
          @param nMsgID message id
          @return pointer to receisved message, NULL for timeout.
       */
      CAnaGateMsg * WaitForMsg( short nMsgID );

      /** Returns a description of the given return code.
         @param nRetCode Return code.
         @return Description of return code.
      */
      static std::string GetErrorMsg( int nRetCode );

      /** Returns a reference to the tcp/ip socket interface.
         @return Connection interface.
      */
      const CAnaTCPIPBase & GetConnection( void ) const;

      /** Returns the device type identifier.
         @return Device type.
      */
      EAnaGateDevice GetDeviceType( void ) const;

      /** Gets the current state of the underlying tcp/ip connection
          @return State enumeration.
      */
      enum CAnaTCPIPBase::EConState CAnaGateClient::GetConnectState( void ) const;

      /** Initialize the interface to the AnaGate partner. 
         @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
      */
      virtual int Open ( void );
      
      /** Sends a close request to the AnaGate partner. 
         @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
      */
      virtual int Close( void );
      
      /** Sends a CMD_ANAGATE_SET_INFO command to the AnaGate device.
         @param pStatusInfo Pointer to the device info structure.
         @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
      */
      int Info( SDeviceInfo * pStatusInfo );
      
      /** Gets status information from the AnaGate device.
         @param sStatusInfo reference to a string, which holds information buffer.
         @remarks The status information can differ for the different devices.
         @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
      */
      int Status( std::string & sStatus );

   protected:
      /** Specialized tcp/ip client connection */ 
      class CConnection : public CAnaTCPIPBase
      {
      public:
         /** Standard-Constructor */
         CConnection( void );
         /** Destructor */
         ~CConnection( void );

         /** Sets parent class to which the DataInd method is routed.
            @param pParentClass Praent Class.
         */
         void SetParentClass( CAnaGateClient * pParentClass );
      private:
         /** This method is called by the base classe, if a data is received via tcp/ip. 
             This overloaded method routes the call just to the DataInd method of the CAnaGateClient class.
             @param pData        Pointer to data buffer.
             @param nDataLen     Length of data buffer.
         */
         virtual void DataInd( const unsigned char *pData,
                               const int            nDataLen );

         virtual void ConInd( const bool   bConnected,
                              const SOCKET oSocket );

         CAnaGateClient *m_pParentClass;  ///< Parent class
      };

      /** Converts return codes from TCP/IP base class. This is neccessary, because the AnaGate device
          has overlapping return code values.
          @param nRC Input return code.
          @return Output return code.
      */
      int ConvertTCPRetCode( int nRC );

      /** Sends a standard AnaGate e telegramm. 
          @param  oMessage AnaGate telegram.
          @return return code (0, if successful).
      */
      virtual int SendMsg( const CAnaGateMsg  & oMessage );
      
      /** Removes old message (confirmations) in message map. 
      */
      void RemoveOldMsgs( void );

      /** Converts a incoming buffer with data to the friendly CAnaGateMsg class.
            @param pData        pointer to data buffer
            @param nDataLen     length of data buffer
            @return Pointter to a newly created CAnaGateMsg obejct, NULL, if error occured.
            @remarks This function is usually called in the DataInd member function.
      */
      CAnaGateMsg * ToMsg( const unsigned char *pData, const int nDataLen );

      /** This method is called by the TCP/IP connection class, if a data telegram is received via tcp/ip.
            @param pData        pointer to data buffer
            @param nDataLen     length of data buffer
      */
      virtual void DataInd( const unsigned char *pData,
                            const int            nDataLen );

      /** This method is called by the TCP/IP connection class, if the connection status is changed.
            @param bConnected  true/false
      */
      virtual void ConInd( const bool bConnected );

      /** This method should be overwritten if the Anagate protocol defines data indications, which 
          has to be responded. In this funcion the respond of the incoming telegram has to be 
          implemented by the derived protocol class.
            @param pMsg  Pointer to the data indication.
            @return true, if message object should not be discarded by base class. 
      */
      virtual bool DataIndRespond( CAnaGateMsg * pMsg );

      /** Retrieves the byte retcode from a standard confirmation.
         @param oMsg Reference to the confirmation telegram.
         @return     Return code.
      */
      virtual int GetCnfRetCode( const CAnaGateMsg & oMsg  );

      /** Reads data from digital io register of the partner. 
         @param nInputBits   Refererence to the variable that receives the digtial input register read.
         @param nOutputBits  Refererence to the variable that receives the digtial output register read.
         @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
      */
      int BaseReadDigital( unsigned long & nInputBits, unsigned long & nOutputBits );

      /** Writes data to digital io register of the DIO partner. 
         @param nOutputBits  Refererence to the variable that hold the digtial IO output register bits to write.
         @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
      */
      int BaseWriteDigital( unsigned long nOutputBits );

      /** Message item contains the message and the receive time */ 
      class CMessageItem
      {
      public:
         /** Standard-Constructor */
         CMessageItem( void );  
         /** Constructor */
         CMessageItem( CAnaGateMsg * pMsg );
         /** Desstructor */
         ~CMessageItem( void );  
         /** Removes the message from the message item container.
            @return     pointer to the message (deletion must be done by caller).
         */
         CAnaGateMsg * PopMsg( void ) ;

         CAnaGateMsg*  m_pMessage;     ///< message 
         CAnaLocalTime m_dtReceived;   ///< message receive time
      };

      typedef std::map<short,CMessageItem*> MessageMap;

      CConnection     m_oTCPIPConnection;  ///< Connection to AnaGate device.

      int            m_nTimeout;          ///< Standard timeout.
      MessageMap     m_aoMsgCnfMap;       ///< Map with incoming messages.
      CAnaMutex      m_oMsgCnfMapMutex;   ///< Mutex for thread-safe access to message map.

      short          m_nCurrentMsgId;     ///< current message id.
      EAnaGateDevice m_nDeviceType;       ///< AnaGate device type.

   private:
      CAnaEvent      m_oMsgCnfReceived;   ///< Event is signalled when incoming confirmation received. 
};

#ifndef USE_DEFAULT_PACKING
// packing mode back to default
#pragma pack(pop)
#endif


#endif // #ifndef _ANAGATE_CLIENT_HPP










