// ----------------------------------------------------------------------------
// Projekt       : Analytica AnaGate API
// File          : AnaGateClientCAN.hpp
// Author        : Axel Schmidt
// Copyright     : (C) 2005 by Analytica GmbH
// ----------------------------------------------------------------------------
// $Id: AnaGateClientCAN.hpp,v 1.20 2008/06/26 07:48:29 tobiashanss Exp $
//
// $Log: AnaGateClientCAN.hpp,v $
// Revision 1.20  2008/06/26 07:48:29  tobiashanss
// Separaten Schalter fr das Packing eingefgt (USE_DEFAULT_PACKING).
//
// Revision 1.19  2008/05/13 12:18:43  stefanwelisch
// zustzliche Funktione Get-/SetZCardParameter hinzugefgt
//
// Revision 1.18  2008/04/18 10:23:39  stefanwelisch
// neue Parameter bei CAN-Kommunikation (Termination, HighSpeed, ConsiderPrio)
//
// Revision 1.17  2008/01/14 17:01:45  axelschmidt
// Callback-Funktion (DataInd) entfernt
//
// Revision 1.16  2007/11/12 10:51:07  axelschmidt
// Byte-Alignment
//
// Revision 1.15  2007/11/08 11:10:20  StefanWelisch
// Defaultwert bei SetGlobals angegeben
//
// Revision 1.14  2007/11/07 12:09:59  StefanWelisch
// In Get/SetGlobals() die Terminierung mit aufgenommen
//
// Revision 1.13  2007/05/21 11:42:47  axelschmidt
// standard timeout 500 -> 1000 ms
//
// Revision 1.12  2005/09/16 09:18:49  axelschmidt
// Digital IO support for I2C and CAN
//
// Revision 1.11  2005/08/15 14:54:49  axelschmidt
// TCP-Protocol changed for global settings
//
// Revision 1.10  2005/08/10 15:46:05  axelschmidt
// GetConfig/SetConfig parameters
//
// Revision 1.9  2005/08/03 16:28:51  axelschmidt
// SetFiter/GetFilter implementation
//
// Revision 1.8  2005/07/22 16:20:36  AxelSchmidt
// open/close are global functions
// get/set config implemented
//
// Revision 1.7  2005/07/22 07:14:48  AxelSchmidt
// new Status-function for all clients
//
// Revision 1.6  2005/07/11 14:34:24  axelschmidt
// only Data-Commands not confirmed
//
// Revision 1.5  2005/07/08 12:03:59  AxelSchmidt
// Baurate is int32
//
// Revision 1.4  2005/06/14 16:09:05  axelschmidt
// CCanMessage removed
//
// Revision 1.3  2005/06/09 16:12:44  axelschmidt
// new Callback-function for received CAN telegrams
//
// Revision 1.2  2005/02/09 12:10:28  axelschmidt
// first implementation for CAN client
//
// Revision 1.1  2005/02/01 08:01:28  AxelSchmidt
// initial
//
// ----------------------------------------------------------------------------

#ifndef _ANAGATE_CLIENT_CAN_HPP
#define _ANAGATE_CLIENT_CAN_HPP

// Includes -------------------------------------------------------------------
#include "AnaGateClient.hpp"
#include "CANMessage.hpp"

//#ifndef WIN32
//#define WINAPI
//#endif
//typedef void (WINAPI * CAN_PF_CALLBACK) ( int nID, const char *pcBuf, int nLen, int nFlags );

#ifndef USE_DEFAULT_PACKING
// packing mode 1 Byte
#pragma pack(push,1)
#endif

/** Standard communication to an device of the AnaGate CAN series via TCP/IP.
    @author  Axel Schmidt
    @version 1.0
*/
class CAnaGateClientCAN : public CAnaGateClient
{
   DECLARE_ANATRACE( CAnaGateClientCAN );

   public:

   enum EEnums
   {
      MAX_FILTER  = 8                  ///< maximum number of filters
   };

   /** Structure that hold the 8 CAN filters */
   typedef struct SConfigFilter
   {
      struct FilterPair
      {
         AnaUInt32 m_nFirst;
         AnaUInt32 m_nSecond;
      };
      struct FilterPair m_oFilter[MAX_FILTER];
   };   

   /** Standard construktor. 
      @param nTimeout Standard timeout for send/receive requests (default = infinite ).
   */
   CAnaGateClientCAN( int nTimeout = 1000 );

   /** Destructor */
   virtual ~CAnaGateClientCAN( void );

   /** Initialize the interface to the I2C partner. 
      @param bConfirmDataTelegrams Set this to true, if the AnaGate shall confirm the telgrams for all data
                               request which are sent to the AnaGate device from client. If telegrams are 
                               received from the AnaGate device, the client confirm the telegrams respectively
                               to this parameter or does not.
      @param bCANMonitor       Set this to true, if the AnaGate shall sent all received CAN messages via data
                               indications to the AnaGate client. 
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int Open ( bool bConfirmDataTelegrams = false, bool bCANMonitor = false );

   /** Sends a CAN telegramm. 
      @param nIdentifier            Id of the sendet message.
      @param pcBuffer               Pointer to the buffer containing the data to be written. 
      @param nNumberOfBytesToWrite  Number of bytes to be written.
      @param nFlags                 CAN message flags (extended/remote format).
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int Write ( AnaUInt32 nIdentifier, const char *pcBuffer, int nNumberOfBytesToWrite, int nFlags = 0);

   /** Sends a CAN telegramm. 
      @param oCANMsg Reference to a initialized CAN-Message class.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int Write ( const CCanMessage & oCANMsg );

   /** Gets the configuration of the AnaGate CAN device.
      @param bConfirmDataTelegrams Is true, if Data reqests and data indications are confirmed by CAN client
                                   and the AnaGate CAN.  
      @param bCANMonitor       True, if the AnaGate shall sent all received CAN messages via data
                               indications to the AnaGate client. 
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int GetConfig( bool & bConfirmDataTelegrams, bool & bCANMonitor );

   /** Sets the configuration of the AnaGate CAN device.
      @param bConfirmDataTelegrams Set this to true, if the AnaGate shall confirm the telgrams for all data
                               request which are sent to the AnaGate device from client. If telegrams are 
                               received from the AnaGate device, the client confirm the telegrams respectively
                               to this parameter or does not.
      @param bCANMonitor       Set this to true, if the AnaGate shall sent all received CAN messages via data
                               indications to the AnaGate client. 
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int SetConfig( bool bConfirmDataTelegrams, bool bCANMonitor );

   /** Gets the filter mask for incoming telegrams.
      @param oFilter Reference to SConfigFilter structure.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int GetFilter( SConfigFilter & oFilter );

   /** Sets the filter mask for incoming telegrams.
      @param oFilter Reference to SConfigFilter structure.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int SetFilter( const SConfigFilter & oFilter );

   /** Sets the globals of the AnaGate CAN device.
      @param nBaudrate      Baud rate.
      @param nOperatingMode Operting mode.
      @param bTermination   Terminatation on/off.
      @param bHighSpeed     HighSpeed     on/off.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int SetGlobals( AnaUInt32 nBaudrate, AnaUInt8 nOperatingMode, bool bTermination = false, bool bHighSpeed = false );

   /** Gets the globals of the AnaGate CAN device.
      @param nBaudrate      Baud rate.
      @param nOperatingMode Operting mode.
      @param bTermination   Terminatation on/off.
      @param bHighSpeed     HighSpeed     on/off.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int GetGlobals( AnaUInt32 & nBaudrate, AnaUInt8 & nOperatingMode, bool & bTermination, bool & bHighSpeed );

   /** Sets the Z-card parameter of the AnaGate CAN device.
      @param nZCardInterval  Z-Card Intervall.
      @param nZCardIDStart   Z-Card Start-ID.
      @param nZCardIDEnd     Z-Card End-ID.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int SetZCardParameter( AnaUInt8 nZCardInterval, AnaUInt16 nZCardIDStart, AnaUInt16 nZCardIDEnd );

   /** Gets the Z-card parameter of the AnaGate CAN device.
      @param nZCardInterval  Z-Card Intervall.
      @param nZCardIDStart   Z-Card Start-ID.
      @param nZCardIDEnd     Z-Card End-ID.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int GetZCardParameter( AnaUInt8 & nZCardInterval, AnaUInt16 & nZCardIDStart , AnaUInt16 & nZCardIDEnd );

   /** Sets the callback function which is called every time a CAN message is received.
      @param pFunction Pointer to the funciton
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
//   void SetCallback( CAN_PF_CALLBACK pFunction );

   /** Reads data from digtial io register of the CAN partner. 
      @param nInputBits   Refererence to the variable that receives the digtial input register read.
      @param nOutputBits  Refererence to the variable that receives the digtial output register read.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int ReadDigital( unsigned long & nInputBits, unsigned long & nOutputBits );

   /** Writes data to digital io register of the CAN partner. 
      @param nOutputBits  Refererence to the variable that hold the digtial IO output register bits to write.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int WriteDigital( unsigned long nOutputBits );

protected:

   /** This method should be overwritten if the Anagate protocol defines data indications, which 
         has to be responded. In this funcion the respond of the incoming telegram has to be 
         implemented by the derived protocol class.
         @param pMsg  Pointer to the data indication.
         @return true, if message object should not be discarded by base class. 
   */
   virtual bool DataIndRespond( CAnaGateMsg * pMsg );

   bool      m_bConfirmDataTelegrams;  ///< Confirmation of data requests and data indications?
//   CAN_PF_CALLBACK m_pfCBFunction;  ///< Callback function
};

#ifndef USE_DEFAULT_PACKING
// packing mode back to default
#pragma pack(pop)
#endif

#endif // #ifndef _ANAGATE_CLIENT_CAN_HPP










