// ----------------------------------------------------------------------------
// Projekt       : Analytica AnaGate API
// File          : AnaGateClientFirmware.hpp
// Author        : Udo Weber
// Copyright     : (C) 2004 by Analytica GmbH
// ----------------------------------------------------------------------------
// $Id: AnaGateClientFirmware.hpp,v 1.6 2008/06/26 07:48:29 tobiashanss Exp $
//
// $Log: AnaGateClientFirmware.hpp,v $
// Revision 1.6  2008/06/26 07:48:29  tobiashanss
// Separaten Schalter fr das Packing eingefgt (USE_DEFAULT_PACKING).
//
// Revision 1.5  2007/11/12 10:50:21  axelschmidt
// Byte-Alignment
//
// Revision 1.4  2007/06/08 14:58:14  axelschmidt
// HW/SW-Versions-Strings
//
// Revision 1.3  2007/05/21 11:43:39  axelschmidt
// standard timeout 500 -> 1000 ms
//
// Revision 1.2  2006/06/19 10:36:46  axelschmidt
// GetInfo/SetIinfo implemented
//
// Revision 1.1  2005/11/17 14:47:38  axelschmidt
// initial
//
// ----------------------------------------------------------------------------

#ifndef _ANAGATE_CLIENT_FIRMWARE_HPP
#define _ANAGATE_CLIENT_FIRMWARE_HPP

// Includes -------------------------------------------------------------------
#include "AnaGateClient.hpp"

#ifndef USE_DEFAULT_PACKING
// packing mode 1 Byte
#pragma pack(push,1)
#endif

/** Standard communication to an device of the AnaGate I2C series via TCP/IP.
    @author  Udo Weber
    @version 1.0
*/
class CAnaGateClientFirmware : public CAnaGateClient
{
   DECLARE_ANATRACE( CAnaGateClientFirmware );

public:
   struct SAnaGateDeviceInfoEx
   {  
      // the following fields are identical to SAnaGateDeviceInfo
      long m_nSWVersion;      ///< Software version (firmware)
      long m_nHWVersion;      ///< Hardware version
      char m_sSerial[4];      ///< Serial number of AnaGate device
      char m_sMACAddr[6];     ///< MAC-Address of AnaGate device
      char m_nDeviceID;       ///< device id of AnaGate device (AnaGate platform)
      char m_nSWDerivateID;   ///< Software derivate id (0 = classic software)
      // the following fields ara new 
      long nIPAnaGate;        ///< ip address of AnaGate device itself
      long nIPGateway;        ///< ip address of gateway
      long nIPDns;            ///< ip address of DNS server
      long nSubnetMask;       ///< sub net mask
      char nDHCP;             ///< DHCP (on/off)
      long nEEPROMVersion;    ///< version eeprom structure
      long nEEPROMSize;       ///< eeprom size

      const std::string GetHWVersionString( void ) const;
      const std::string GetSWVersionString( void ) const;
   };

   /** Standard construktor. 
      @param nTimeout Standard timeout for send/receive requests (default = infinite ).
   */
   CAnaGateClientFirmware( int nTimeout = 1000 );

   /** Destructor */
   virtual ~CAnaGateClientFirmware( void );

   /** Initialize the interface to the I2C partner. 
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int Open ( int &nRandomKeyFromAnaGate, int &nSWVersion );

   /** Sends the key back to the AnaGate
      @param nRandomKeyFromAnaGate         Key received from the AnaGate in the OpenConfirm message
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int SendKey( unsigned long nRandomKeyFromAnaGate );


   int Restart( void );

   /** Reads data from the flash of the AnaGate. Always 2k (2048 Bytes) are read with this command.
      @remarks Bit 0 if the slave address (the read/write flag) is always set by the AnaGate device itself.
      @param nBlock                Index of 2k Block (0 .. 127) to be read
      @param pcBuffer              Pointer to the buffer that receives the data read. 
      @param nNumberOfBytesRead    Refererence to the variable that receives the number of bytes read.
                                   #Read sets this value to zero before doing any work or error checking.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int ReadFlash ( unsigned short nBlock, char *pcBuffer, int & nNumberOfBytesRead  );

   /** Writes data to the flash of the AnaGate. Always 2k (2048 Bytes) must be written with this command.
      @param nBlock                 Index of 2k Block (0 .. 127) to be read
      @param pcBuffer               Pointer to the buffer containing the data to be written. 
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int WriteFlash ( unsigned short nBlock, const char *pcBuffer );

   /** Clears the whole flash of the AnaGate
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int ClearFlash ( void );

   /** Returns the size of the internal EEPROM of the AnaGate
      @param nNumberOfBytesRead    Refererence to the variable that receives the number of bytes of the EEPROM.
                                   #GetEEPromInfo sets this value to zero before doing any work or error checking.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int GetEEPromInfo ( int &nEEPROMSize );

   /** Reads data from the specified offset to a data buffer. If requested data length is greater
       than the actual page size, the data length is reduced to page size.
      @remarks This command is a specialized version of the #Read method and is optimized for I2C EEProms.
               <p>This version implicit performs a write request to set the offset where to read from and the
               following read request in a single tcpip telegramm.</p>
               <p>Use this method only for EEProm partners</p>
      @param nOffset               Offset position where data is to be read (EEProm partner). 
      @param pcBuffer              Pointer to the buffer that receives the data read. 
      @param nNumberOfBytesToRead  Number of bytes to be read.
      @param nNumberOfBytesRead    Refererence to the variable that receives the number of bytes read.
                                   #Read sets this value to zero before doing any work or error checking.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */

   int ReadEEProm ( AnaUInt32 nOffset, char *pcBuffer, int nNumberOfBytesToRead, int & nNumberOfBytesRead );

   /** Writes data to the specified offset. 
      @remarks This command is a specialized version of the #Write method and is optimized for I2C EEProms.
               <p>Use this method only for EEProm partners</p>
      @param nOffset               Offset position where data is to be written (EEProm partner). 
      @param pcBuffer              Pointer to the buffer containing the data to be written. 
      @param nNumberOfBytesToWrite Number of bytes to be written.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int WriteEEProm( AnaUInt32 nOffset, const char *pcBuffer, int nNumberOfBytesToWrite );
   
   /** Sends a reset request to the I2C bus.
       @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int Reset( void );

   /** Reads all device informations from the internal eeprom of the anagate device. 
       @param oInfo Reference to the device info structure.
       @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int GetInfo( struct SAnaGateDeviceInfoEx & oInfo );

   /** Sets all device informations from the internal eeprom of the anagate device. This command should be used very carefully.
       @param oInfo Reference to the device info structure.
       @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int SetInfo( struct SAnaGateDeviceInfoEx & oInfo );

 protected:
   /** Converts the offset into a char format depending on actual offset format.
      @param nOffset       The offset.
      @param nFormatFlags  Address format flags (see #EOffsetFormat).
      @return the char    Buffer aequivalent of the offset.
   */
   std::string ToOffset( AnaUInt32 nOffset, AnaUInt32 nFormatFlags );
};

#ifndef USE_DEFAULT_PACKING
// packing mode back to default
#pragma pack(pop)
#endif

#endif // #ifndef _ANAGATE_CLIENT_FIRMWARE_HPP










