// ----------------------------------------------------------------------------
// Projekt       : Analytica AnaGate API
// File          : AnaGateClientI2C.hpp
// Author        : Axel Schmidt
// Copyright     : (C) 2004 by Analytica GmbH
// ----------------------------------------------------------------------------
// $Id: AnaGateClientI2C.hpp,v 1.17 2008/11/18 16:20:38 stefanwelisch Exp $
//
// $Log: AnaGateClientI2C.hpp,v $
// Revision 1.17  2008/11/18 16:20:38  stefanwelisch
// neue Geschwindigkeiten 1Mbit/s und  3,4Mbit/s hinzugefgt
//
// Revision 1.16  2008/06/26 07:48:29  tobiashanss
// Separaten Schalter fr das Packing eingefgt (USE_DEFAULT_PACKING).
//
// Revision 1.15  2008/03/04 14:31:58  axelschmidt
// support for 24xx1025 (microchip)
//
// Revision 1.14  2007/11/12 10:50:21  axelschmidt
// Byte-Alignment
//
// Revision 1.13  2007/05/21 11:42:47  axelschmidt
// standard timeout 500 -> 1000 ms
//
// Revision 1.12  2005/11/30 16:16:04  axelschmidt
// Sequence command implemented
//
// Revision 1.11  2005/11/28 17:07:47  axelschmidt
// errorbyte parameter for I2CWrite function
//
// Revision 1.10  2005/09/16 09:18:48  axelschmidt
// Digital IO support for I2C and CAN
//
// Revision 1.9  2005/07/22 16:21:59  AxelSchmidt
// open/close are global functions
// get/set config implemented
// baudrate 32 bits now
//
// Revision 1.8  2005/07/22 07:14:38  AxelSchmidt
// new Status-function for all clients
//
// Revision 1.7  2004/08/09 15:49:29  AxelSchmidt
// div. Parameter von signed auf unsigned umgesetzt
//
// Revision 1.6  2004/08/09 06:58:33  AxelSchmidt
// offset address format changed
//
// Revision 1.5  2004/08/06 07:01:20  AxelSchmidt
// Reset-Command hat keine Parameter
//
// Revision 1.4  2004/08/05 16:09:15  AxelSchmidt
// neue Funktion GetEEPROMSlaveAddress
//
// Revision 1.3  2004/08/02 07:08:27  AxelSchmidt
// DeviceSelector wird nicht mehr in der Klasse selbst gespeichert
//
// Revision 1.2  2004/07/21 16:14:22  AxelSchmidt
// Offset format for EEProm commands not longer stored in class.
//
// Revision 1.1  2004/07/01 16:28:58  AxelSchmidt
// initial version
//
// ----------------------------------------------------------------------------

#ifndef _ANAGATE_CLIENT_I2C_HPP
#define _ANAGATE_CLIENT_I2C_HPP

// Includes -------------------------------------------------------------------
#include "AnaGateClient.hpp"

#ifndef USE_DEFAULT_PACKING
// packing mode 1 Byte
#pragma pack(push,1)
#endif

/** Standard communication to an device of the AnaGate I2C series via TCP/IP.
    @author  Axel Schmidt
    @version 1.0
*/
class CAnaGateClientI2C : public CAnaGateClient
{
   DECLARE_ANATRACE( CAnaGateClientI2C );

public:
   /** Decribes the length of the offset field for read/write requests. */
   enum EOffsetFormat
   {
      E2E1E0   = 0x0,      ///< no chip enable bits are used for address extension

      E2E1A0   = 0x0100,   ///< offset address uses chip enable bit E0 in slave address.
      E2A1A0   = 0x0200,   ///< offset address uses chip enable bit E0 and E1 in slave address.
      A2A1A0   = 0x0300,   ///< offset address uses chip enable bits E0, E1 and E2 in slave address .

      A0E1E0   = 0x0500,   ///< offset address use chip enable bits E2.
      A1A0E0   = 0x0600,   ///< offset address use chip enable bits E2 and E1 in slave address.

      bit8     = 0x0008,   ///< offset address is a 8 bit field
      bit16    = 0x000F,   ///< offset address is a 16 bit field
   };

   /** Supported baudrates for I2C bus */
   enum EBaudrate
   {
      KB100_OLD  = 100,       ///< 100 kilo bit
      KB400_OLD  = 400,       ///< 400 kilo bit
      KB50       =   50000,    ///<  50 kilo bit
      KB100      =  100000,    ///< 100 kilo bit
      KB200      =  200000,    ///< 200 kilo bit
      KB400      =  400000,    ///< 400 kilo bit
      KB1000     = 1000000,    ///< 1000 kilo bit
      KB3400     = 3400000     ///< 3400 kilo bit
   };

   /** Describes an I2C command. Read/Write bit must be set in slave address. */ 
   struct I2CCommand
   {
      AnaUInt16 m_nSlaveAddress;       ///< slave address
      AnaUInt16 m_nBufferLen;          ///< buffer length
      char    * m_pReadWriteBuffer;    ///< pointer to byte buffer
   };

   typedef std::vector<I2CCommand> I2CSequence;

   /** Standard construktor. 
      @param nTimeout Standard timeout for send/receive requests (default = infinite ).
   */
   CAnaGateClientI2C( int nTimeout = 1000 );

   /** Destructor */
   virtual ~CAnaGateClientI2C( void );

   /** Initialize the interface to the I2C partner. 
      @param nBaudrate   Baud rate of bus.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int Open ( AnaUInt32 nBaudrate = KB400 );

   /** Reads data from the I2C partner. The data offset has to be set by a seperate write command. 
      @remarks Bit 0 if the slave address (the read/write flag) is always set by the AnaGate device itself.
      @param nSlaveAddress         Slave address.
      @param pcBuffer              Pointer to the buffer that receives the data read. 
      @param nNumberOfBytesToRead  Number of bytes to be read.
      @param nNumberOfBytesRead    Refererence to the variable that receives the number of bytes read.
                                   #Read sets this value to zero before doing any work or error checking.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int Read( AnaUInt16 nSlaveAddress, char *pcBuffer, int nNumberOfBytesToRead, int & nNumberOfBytesRead );

   /** Writes data to the I2C partner The data is routed to the partner directly without any transformation,
      so the user of the interface has to build a correct data buffer. 
      @remarks Bit 0 if the slave address (the read/write flag) is always set by the AnaGate device itself.
      @param nSlaveAddress          Slave address.
      @param pcBuffer               Pointer to the buffer containing the data to be written. 
      @param nNumberOfBytesToWrite  Number of bytes to be written.
      @param nByteNumberLastError   If an error occurs then this is set to the byte number which produces the 
                                    error. If slave address produces the error it is set to zero.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int Write ( AnaUInt16 nSlaveAddress, const char *pcBuffer, int nNumberOfBytesToWrite, int & nByteNumberLastError );

   /** Writes an I2C command sequence to the AnaGate I2C. The Sequence is an vector of read and
       write commands.
      @param oaCmdArray             Array with the single read/write requests. 
      @param nByteNumberLastError   If an error occurs then this is set to the byte number which produces the 
                                    error. If slave address produces the error it is set to zero.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int WriteSequence ( I2CSequence & oaCmdArray, int & nByteNumberLastError );

   /** Writes an I2C command sequence to the AnaGate I2C. The Sequence is an array of read and
       write commands.
      @param pCmdArray              Pointer to an array of I2C commands. 
      @param nArrayLen              Number of items in the array of I2C commands. 
      @param nByteNumberLastError   If an error occurs then this is set to the byte number which produces the 
                                    error. If slave address produces the error it is set to zero.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int WriteSequence ( I2CCommand * pCmdArray, int nArrayLen, int & nByteNumberLastError );
   
   /** Writes an I2C command sequence to the AnaGate I2C. The sequence already converted to a character buffer.
      @param pcWriteBuffer          Pointer to the buffer containing the data to be written. 
      @param nNumberOfBytesToWrite  Number of bytes to be written.
      @param pcReadBuffer           Pointer to the buffer that receives the data read. 
      @param nNumberOfBytesToRead   Number of bytes to be read.
      @param nNumberOfBytesRead     Refererence to the variable that receives the number of bytes read.
      @param nByteNumberLastError   If an error occurs then this is set to the byte number which produces the 
                                    error. If slave address produces the error it is set to zero.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int WriteSequence ( const char *pcWriteBuffer, int nNumberOfBytesToWrite, 
                       char *pcReadBuffer, int nNumberOfBytesToRead, int & nNumberOfBytesRead, 
                       int & nByteNumberLastError );

   /** Reads data from the specified offset to a data buffer. If requested data length is greater
       than the actual page size, the data length is reduced to page size.
      @remarks This command is a specialized version of the #Read method and is optimized for I2C EEProms.
               <p>This version implicit performs a write request to set the offset where to read from and the
               following read request in a single tcpip telegramm.</p>
               <p>Use this method only for EEProm partners</p>
      @param nChipAddress          Chip enable address (bit 1 - 3 of the slave address).
                                   Other Bits of the parameter are ignored, the device type identifier (bit 4 - 7) for
                                   EEPROMs is generated by the #Read method itself.
      @param nOffset               Offset position where data is to be read (EEProm partner). 
      @param nFormatFlags          Address format flags (see #EOffsetFormat).
      @param pcBuffer              Pointer to the buffer that receives the data read. 
      @param nNumberOfBytesToRead  Number of bytes to be read.
      @param nNumberOfBytesRead    Refererence to the variable that receives the number of bytes read.
                                   #Read sets this value to zero before doing any work or error checking.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */

   int ReadEEProm ( AnaUInt16 nChipAddress, AnaUInt32 nOffset, AnaUInt32 nFormatFlags, 
                    char *pcBuffer, int nNumberOfBytesToRead, int & nNumberOfBytesRead );

   /** Writes data to the specified offset. 
      @remarks This command is a specialized version of the #Write method and is optimized for I2C EEProms.
               <p>Use this method only for EEProm partners</p>
      @param nChipAddress          Chip enable address (bit 1 - 3 of the slave address).
                                   Other Bits of the parameter are ignored, the device type identifier (bit 4 - 7) for
                                   EEPROMs is generated by the #Read method itself.
      @param nOffset               Offset position where data is to be written (EEProm partner). 
      @param nFormatFlags          Address format flags (see #EOffsetFormat).
      @param pcBuffer              Pointer to the buffer containing the data to be written. 
      @param nNumberOfBytesToWrite Number of bytes to be written.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int WriteEEProm( AnaUInt16 nChipAddress, AnaUInt32 nOffset, AnaUInt32 nFormatFlags, 
                    const char *pcBuffer, int nNumberOfBytesToWrite );
   
   /** Sends a reset request to the I2C bus.
       @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int Reset( void );

   /** Sets the baudrate of the AnaGate I2C device.
      @param nBaudrate Baud rate.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int SetBaudrate( AnaUInt32 nBaudrate );

   /** Gets the current baudrate of the AnaGate I2C device.
      @param nBaudrate Baud rate.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int GetBaudrate( AnaUInt32 & nBaudrate );

   /** Reads data from digtial io register of the I2C partner. 
      @param nInputBits   Refererence to the variable that receives the digtial input register read.
      @param nOutputBits  Refererence to the variable that receives the digtial output register read.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int ReadDigital( unsigned long & nInputBits, unsigned long & nOutputBits );

   /** Writes data to digital io register of the I2C partner. 
      @param nOutputBits  Refererence to the variable that hold the digtial IO output register bits to write.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int WriteDigital( unsigned long nOutputBits );

   /** Computes the EEPROM slave address, which is to be used for reading/writing for the given data offset
       depending on the data offset address format.
       @remarks The slave address for a EEPROM is always a 7 bit address (bits 1-7). Bit 0 is the read/write flag
                and is always set by the AnaGate device itself. The device type idenitifier (bits 4-7) is
                for EEPROMs fix (b'1010'), so only the chip enable address (bits 1-3, also named E0, E1 and E2) 
                can be used for addressing the memory device.
                <p>Some I2C memory devices are using the chip enable address bits E0, E1 and E2 for extended offset length. 
                For example the ST24C04 EEPROM has 512 data bytes and is addressed via a 8 bit offset adress and bit E0 in the 
                slave address. For this device only chip addresses from 0 to 3 are valid (bits E2 and E2).  
                </p>
      @param nChipAddress  Chip enable address (is mapped to bit 1 - 3 of the slave address).
      @param nOffset       Offset position where data is to be written (EEProm partner). 
      @param nFormatFlags  Address format flags (see #EOffsetFormat).
      @return Slave address.
  */
   static AnaUInt16 GetEEPROMSlaveAddress( AnaUInt16 nChipAddress, AnaUInt32 nOffset, AnaUInt32 nFormatFlags );

protected:
   /** Converts the offset into a char format depending on actual offset format.
      @param nOffset       The offset.
      @param nFormatFlags  Address format flags (see #EOffsetFormat).
      @return the char    Buffer aequivalent of the offset.
   */
   std::string ToOffset( AnaUInt32 nOffset, AnaUInt32 nFormatFlags );

private:
   AnaUInt32 m_nBaudrate;      ///< baud rate.
};

#ifndef USE_DEFAULT_PACKING
// packing mode back to default
#pragma pack(pop)
#endif

#endif // #ifndef _ANAGATE_CLIENT_I2C_HPP










