// ----------------------------------------------------------------------------
// Projekt       : Analytica AnaGate API
// File          : AnaGateClientSPI.hpp
// Author        : Axel Schmidt
// Copyright     : (C) 2004 by Analytica GmbH
// ----------------------------------------------------------------------------
// $Id: AnaGateClientSPI.hpp,v 1.6 2008/06/26 07:48:29 tobiashanss Exp $
//
// $Log: AnaGateClientSPI.hpp,v $
// Revision 1.6  2008/06/26 07:48:29  tobiashanss
// Separaten Schalter fr das Packing eingefgt (USE_DEFAULT_PACKING).
//
// Revision 1.5  2007/11/12 10:50:20  axelschmidt
// Byte-Alignment
//
// Revision 1.4  2007/05/21 11:42:47  axelschmidt
// standard timeout 500 -> 1000 ms
//
// Revision 1.3  2006/02/03 17:54:22  axelschmidt
// ToOffset function changed
//
// Revision 1.2  2005/12/27 08:26:28  AxelSchmidt
// base functions of SPI interface
//
// Revision 1.1  2005/11/28 17:05:48  axelschmidt
// SPI integrated, initial version
//
// ----------------------------------------------------------------------------

#ifndef _ANAGATE_CLIENT_SPI_HPP
#define _ANAGATE_CLIENT_SPI_HPP

// Includes -------------------------------------------------------------------
#include "AnaGateClient.hpp"

#ifndef USE_DEFAULT_PACKING
// packing mode 1 Byte
#pragma pack(push,1)
#endif

/** Standard communication to an device of the AnaGate SPI series via TCP/IP.
    @author  Axel Schmidt
    @version 1.0
*/
class CAnaGateClientSPI : public CAnaGateClient
{
   DECLARE_ANATRACE( CAnaGateClientSPI );

public:
   /** Decribes the length of the offset field for read/write requests. */
   enum EOffsetFormat
   {
      bit8     = 0x0001,   ///< offset address is a 8 bit field
      bit16    = 0x0002,   ///< offset address is a 16 bit field
      bit24    = 0x0003,   ///< offset address is a 24 bit field
      bit32    = 0x0004    ///< offset address is a 32 bit field
   };

   /** Supported baudrates for SPI bus */
   enum EBaudrate
   {
      KB100      = 100000,    ///< 100 kilo bit
      KB400      = 400000     ///< 400 kilo bit
   };

   /** Standard construktor. 
      @param nTimeout Standard timeout for send/receive requests (default = infinite ).
   */
   CAnaGateClientSPI( int nTimeout = 1000 );

   /** Destructor */
   virtual ~CAnaGateClientSPI( void );

   /** Initialize the interface to the SPI partner. 
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int Open ( void );

   /** Exchanges data between the SPI partner and the remote pc. 
      @param pcBufWrite    Pointer to the buffer containing the data to be written. 
      @param nBufWriteLen  Number of bytes to be written.
      @param pcBufRead     Pointer to the buffer that receives the data read. 
      @param nBufReadLen   Number of bytes to be read (should be same as number of bytes written).
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int DataReq( const char * pcBufWrite, int nBufWriteLen, char * pcBufRead, int nBufReadLen );

   /** Sends a reset request to the SPI bus.
       @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int Reset( void );

   /** Sets the global device settings of the AnaGate SPI device.
       @param nBaudrate Baud rate.
       @param nSigLevel Signal level.
       @param nAuxVoltage Auxiliary voltage supply.
       @param nClockMode The phase and the polarity of the clock line.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int SetGlobals( AnaUInt32 nBaudrate, AnaUInt8 nSigLevel, AnaUInt8 nAuxVoltage, AnaUInt8 nClockMode );

   /** Gets the current global device settings of the AnaGate SPI device.
       @param nBaudrate Baud rate.
       @param nSigLevel Signal level.
       @param nAuxVoltage Auxiliary voltage supply.
       @param nClockMode The phase and the polarity of the clock line.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int GetGlobals( AnaUInt32 & nBaudrate, AnaUInt8 & nSigLevel, AnaUInt8 & nAuxVoltage, AnaUInt8 & nClockMode );

   /** Reads data from digtial io register of the SPI partner. 
      @param nInputBits   Refererence to the variable that receives the digtial input register read.
      @param nOutputBits  Refererence to the variable that receives the digtial output register read.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int ReadDigital( unsigned long & nInputBits, unsigned long & nOutputBits );

   /** Writes data to digital io register of the SPI partner. 
      @param nOutputBits  Refererence to the variable that hold the digtial IO output register bits to write.
      @return. If the function succeeds, the return value is zero. If the function fails, the return value is non zero. 
   */
   int WriteDigital( unsigned long nOutputBits );

   /** Computes the EEPROM slave address, which is to be used for reading/writing for the given data offset
       depending on the data offset address format.
       @remarks The slave address for a EEPROM is always a 7 bit address (bits 1-7). Bit 0 is the read/write flag
                and is always set by the AnaGate device itself. The device type idenitifier (bits 4-7) is
                for EEPROMs fix (b'1010'), so only the chip enable address (bits 1-3, also named E0, E1 and E2) 
                can be used for addressing the memory device.
                <p>Some SPI memory devices are using the chip enable address bits E0, E1 and E2 for extended offset length. 
                For example the ST24C04 EEPROM has 512 data bytes and is addressed via a 8 bit offset adress and bit E0 in the 
                slave address. For this device only chip addresses from 0 to 3 are valid (bits E2 and E2).  
                </p>
      @param nChipAddress  Chip enable address (is mapped to bit 1 - 3 of the slave address).
      @param nOffset       Offset position where data is to be written (EEProm partner). 
      @param nFormatFlags  Address format flags (see #EOffsetFormat).
      @return Slave address.
  */
   static AnaUInt16 GetEEPROMSlaveAddress( AnaUInt16 nChipAddress, AnaUInt32 nOffset, AnaUInt32 nFormatFlags );

public:
   /** Converts the offset into a char format depending on actual offset format.
      @param nOffset      The offset.
      @param nFormat      Offset format (8, 16, 24 or 32 bit.
      @return the char    Buffer aequivalent of the offset.
   */
   static std::string ToOffset( AnaUInt32 nOffset, EOffsetFormat nFormat );

private:
   AnaUInt32 m_nBaudrate;      ///< baud rate.
};

#ifndef USE_DEFAULT_PACKING
// packing mode back to default
#pragma pack(pop)
#endif

#endif // #ifndef _ANAGATE_CLIENT_SPI_HPP










