// ----------------------------------------------------------------------------
// Projekt       : Analytica AnaGate API
// File          : DeviceManager.h
// Author        : Axel Schmidt
// Copyright     : (C) 2004 by Analytica GmbH
// ----------------------------------------------------------------------------
// $Id: DeviceManager.h,v 1.8 2008/05/13 12:20:06 stefanwelisch Exp $
//
// $Log: DeviceManager.h,v $
// Revision 1.8  2008/05/13 12:20:06  stefanwelisch
// DevicePort bei OpenDevice hinzugefgt
//
// Revision 1.7  2006/06/20 12:43:35  axelschmidt
// make it thread-safe
//
// Revision 1.6  2005/07/22 16:13:40  AxelSchmidt
// autoset port
//
// Revision 1.5  2005/07/11 16:17:25  axelschmidt
// .
//
// Revision 1.4  2004/08/13 12:50:20  AxelSchmidt
// Error in GetDevice-template
//
// Revision 1.3  2004/07/23 15:52:54  AxelSchmidt
// new method to free all resources
//
// Revision 1.2  2004/07/21 16:12:21  AxelSchmidt
// new Timeout-Parameter in OpenDevice
//
// Revision 1.1  2004/07/21 06:53:47  AxelSchmidt
// initial version
//
// ----------------------------------------------------------------------------

#ifndef _DEVICE_MANAGER_H
#define _DEVICE_MANAGER_H

// Includes -------------------------------------------------------------------
#include <AnaGate.h>
#include <AnaMutex.h>
#include "AnaGateErrors.h"
#include <map>

// Defines --------------------------------------------------------------------
// Prototyping ----------------------------------------------------------------

class CAnaTraceServer;
class CAnaGateClient;

// Classes --------------------------------------------------------------------

/** Describes the communication structure of a AnaGate device which is allocated
    to a device handle. */
class CDevice
{
public:
   ~CDevice();
   
   CAnaGateClient*     m_pClient;
   std::string         m_sIPAddress;
   int                 m_nPort;
   int                 m_nHandle;
   int                 m_nDeviceType;
};


/** Manages the device handles which are requested through the DLL-interface of the AnaGate API. */
class CDeviceManager
{
   typedef std::map<int,CDevice*> HandleMap;
   
public:
   /** Opens an AnaGate device.
      @param hHandle Device handle (from a successfull #OpenDevice call).
      @param pClient Stores a pointer to the device tcp/ip client object, which can be used to
                     perform all device specific commands.
      @returns Returns 0 if successful.  
   */
   template<class C> static int GetClient( int hHandle, C ** pClient);

   /** Opens an AnaGate device.
      @param pHandle Pointer to a handle, in which the device handle is stored, if device is
                     opened successfully.
      @param pcIPAddress Tcp/ip address of the AnaGate device.
      @param nTimeout    Standard tcp/ip time out in milliseconds.
      @param nDeviceType Device type selector. 
      @param nDevicePort Device port selector. 
      @returns Returns 0 if successful.  
   */
   static int OpenDevice( int *pHandle, const char * pcIPAddress, int nTimeout, int nDeviceType, int nDevicePort = 0 );

   /** Closes an open AnaGate device.
      @param hHandle Device handle (from a successfull #OpenDevice call).
      @returns Returns 0 if successful.  
   */
   static int CloseDevice( int hHandle );

   /** Stops the trace server */
   static void StopTraceServer( void );

   /** Starts the trace server */
   static void StartTraceServer( void );

   /** Closes all aktive handles und frees all allocated resources */
   static void CloseAllDevices( void );
protected:

   /** Returns the device handle structure of the given device handle.
      @param hHandle Device handle (from a successfull #OpenDevice call).
      @returns Pointer to the matching device handle structure or NULL, if not found.  
   */
   static CDevice * GetDevice( int hHandle );

   static CAnaTraceServer *m_pTraceServer;   ///< pointer to the one and only trace server
   static HandleMap        m_aoHandleMap;    ///< map with all used device handles
   static CAnaMutex        m_oMutexHandleMap;///< mutex for thread safe map access
   static int              m_nMaxHandle;     ///< next free device handle
};

// Additional class templates -------------------------------------------------

template<class D> int CDeviceManager::GetClient( int hHandle, D ** pClient)
{
   int nRC = ERR_INVALID_DEVICE_HANDLE;
   CDevice *pDevice = GetDevice( hHandle );
   if ( pDevice )
   {
      *pClient = static_cast<D*>(pDevice->m_pClient);
      nRC = 0;
   }
   return nRC;
}

#endif // #ifndef _DEVICE_MANAGER_H
